%[g,I,H,t,T]=flex02d01(m,b,f)
%----------------------------
%
%USB interface to the Flex02-01D/C correlator.
%
%     Marcel Leutenegger  11.4.2011
%
%Input:
% m   's'    Reset for (AxA) or (AxB)
%     'd'    Reset for (AxA & BxB) or (AxB & BxA)
%     'q'    Reset for (AxA & AxB & BxB & BxA)
%     'a'    Start a new auto-correlation
%     'c'    Start a new cross-correlation
%     'e'    Stop the correlation (default)
% b   Bin time for intensity histograms [s] {1us}
% f   File name for photon traces {none}
%
%Output:
% g   Correlation(s) (1120x1), (624x2) or (288x4)
% I   Intensity traces in both channels (Nx2) [Hz]
% H   Intensity histograms (2048x2)
% t   Delays (1120x1), (624x1) or (288x1) [s]
% T   Elapsed time since last start [s]
%
function [g,I,H,t,T]=flex02d01(m,b,f)
persistent Nf Td Tg Tt Pt n s;
persistent tg ti;
if nargin
   switch m
   case {'s','S'}
      n=1;
      Nf=[];
      Tg=delays(34,32);
   case {'d','D'}
      n=2;
      Nf=[];
      Tg=delays(38,16);
   case {'q','Q'}
      n=4;
      Nf=[];
      Tg=delays(35,8);
   case {'a','A','c','C'}
      if ~numel(Tg)
         n=1;
         Tg=delays(34,32);
      end
      s=size(Tg);
      Nf=repmat(1./max(0.2,0.5 + 0.1*randn(1,n)),s);
      Td=repmat(1./max(40e-6,120e-6 + 15e-6*randn(1,n)),s);
      Tt=repmat(1./max(0.5e-6,2e-6 + 0.1e-6*randn(1,n)),s);
      Pt=max(0.05,0.2 + 0.05*randn(1,n));
      Pt=repmat(Pt./(1-Pt),s);
      tg=curtime;
      ti=tg;
      s=[];
   case {'e','E'}
      if ~numel(s)
         s=curtime;
      end
   otherwise
      n=[];
   end
end
if nargout
   if ~numel(n*numel(Nf))
      error('Correlator not initialized.');
   end
   if ~numel(s)
      T=curtime-tg;
   else
      T=s-tg;
   end
   t=repmat(Tg,1,n);
   g=Nf.*((1 + t.*Td).^(-1.5) + Pt.*exp(-t.*Tt));
   g=1 + g.*(1 + 0.05/(1+T).*randn(size(g)));
   if nargout > 1
      dt=pow2(1/640e6,25);
      if ~numel(s)
         ni=floor((curtime-ti)/dt);
      else
         ni=floor((s-ti)/dt);
      end
      I=repmat([220e3 150e3],ni,1) + 30e3.*randn(ni,2);
      ti=ti + ni*dt;
   end
   if nargout > 2
      H=repmat((0:2047).',1,2);
      H=poisspdf(H,repmat(20 + randn(1,2)/(1+T),2048,1));
      H(isnan(H) | H < 1e-6)=0;
      H=H./repmat(sum(H),2048,1);
   end
   if nargout > 3
      t=Tg;
   end
end


function t=delays(m,n)
t=pow2(repmat(1/640e6*(1+n:2*n).',1,m),repmat(0:m-1,n,1));
t=[1/640e6*(1:n).';t(:)];
